local mod = ...



-- Building Asset Processor -------------------------------------------------------------------------------------
mod:registerAssetProcessor("models/Rubbish1.fbx", { DataType = "BUILDING_ASSET_PROCESSOR" })




-- *************************************************************************************************************
-- Rubbish -------------------------------------------------------------------------------------
-- A non-functioning component that is used by the Fire mod to find and burn rubbish
-- *************************************************************************************************************

local COMP_RUBBISH = {
	TypeName = "COMP_RUBBISH",
	ParentType = "COMPONENT",
	Properties = {
    }
}

-- Register custom Class
mod:registerClass(COMP_RUBBISH)



-- Rubbish Prefabs -------------------------------------------------------------------------------------
local listRubbishPrefab = {}

local function registerRubbish(mesh)

    -- create prefab
    mod:registerAssetId("models/Rubbish1.fbx/Prefab/" .. mesh, "PREFAB_RUBBISH_" .. mesh)

    -- ground
    mod:registerPrefabComponent("PREFAB_RUBBISH_" .. mesh, {
        DataType = "COMP_GROUNDED",
        AutoDisable = true,
    })

    -- rubbish component (used by the fire mod to find rubbish and burn it)
    mod:registerPrefabComponent("PREFAB_RUBBISH_" .. mesh, { DataType = "COMP_RUBBISH" } )

    -- resource container
    mod:registerPrefabComponent( "PREFAB_RUBBISH_" .. mesh, {
        DataType = "COMP_RESOURCE_CONTAINER",
        ResourceData = "RESOURCE_RUBBISH",
        IsReplenishable = false,
        ResourceValue = 0.1, -- how many are gathered with each pick
        AvailableQuantity = 1, -- maximum container stock
        Radius = 1.0, -- distance from container that the villager will stand to pick
        IsDestroyWhenEmpty = true,
    })

    -- record in prefab list for use later
    table.insert(listRubbishPrefab, "PREFAB_RUBBISH_" .. mesh)

end

registerRubbish("BARREL1")
registerRubbish("BARREL2")
registerRubbish("BARREL3")
registerRubbish("BARREL4")
registerRubbish("BARREL5")

-- registerRubbish("BASKET1")
-- registerRubbish("BASKET2")
-- registerRubbish("BASKET3")

registerRubbish("BOX1")
registerRubbish("BOX2")
registerRubbish("BOX3")

registerRubbish("BUCKET1")
registerRubbish("BUCKET2")

registerRubbish("CHAIR1")
registerRubbish("CHAIR2")
registerRubbish("CHAIR3")
registerRubbish("CHAIR4")
registerRubbish("CHAIR5")

registerRubbish("STONE1")
registerRubbish("STONE2")
registerRubbish("STONE3")

registerRubbish("BRICK1")
registerRubbish("BRICK2")
registerRubbish("BRICK3")

registerRubbish("POTTERY1")
registerRubbish("POTTERY2")
registerRubbish("POTTERY3")



-- Building: Rubbish Collector -------------------------------------------------------------------------------------
mod:registerAssetId("models/Rubbish1.fbx/Prefab/Rubbish.Collector.Part", "PREFAB_RUBBISH_COLLECTOR")

mod:registerPrefabComponent("models/Rubbish1.fbx/Prefab/Rubbish.Collector.Part/Stool.001", { DataType = "COMP_GROUNDED" })
mod:registerPrefabComponent("models/Rubbish1.fbx/Prefab/Rubbish.Collector.Part/Pottery.001", { DataType = "COMP_GROUNDED" })
mod:registerPrefabComponent("models/Rubbish1.fbx/Prefab/Rubbish.Collector.Part/Pottery.002", { DataType = "COMP_GROUNDED" })
mod:registerPrefabComponent("models/Rubbish1.fbx/Prefab/Rubbish.Collector.Part/Barrel.001", { DataType = "COMP_GROUNDED" })
mod:registerPrefabComponent("models/Rubbish1.fbx/Prefab/Rubbish.Collector.Part/Box.001", { DataType = "COMP_GROUNDED" })
mod:registerPrefabComponent("models/Rubbish1.fbx/Prefab/Rubbish.Collector.Part/Box.002", { DataType = "COMP_GROUNDED" })


mod:registerAsset({
	DataType = "BUILDING",
	Id = "BUILDING_RUBBISH_COLLECTOR",
	Name = "BUILDING_RUBBISH_COLLECTOR_NAME",
	Description = "BUILDING_RUBBISH_COLLECTOR_DESC",
	BuildingType = "RESOURCE_PRODUCTION",
    AssetCoreBuildingPart = "BUILDING_PART_RUBBISH_COLLECTOR",
    VillagerRequired = {
        Status = "NEWCOMER",
        Quantity = 10,
    },
})

mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_RUBBISH_COLLECTOR",
	Name = "BUILDING_PART_RUBBISH_COLLECTOR_NAME",
    AssetBuildingFunction = "BUILDING_FUNCTION_RUBBISH_COLLECTOR",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "PREFAB_RUBBISH_COLLECTOR",
        MiniatureConfig = {
            ClipUnderGround = true,
            GroupHeight = 0.0,
            CameraPosition = { 0, 0.8, -11 },
            OrientationOffset = 0.0,
            CameraPitchOffset = 10.0
        },
	},
	BuildingZone = {
        ZoneEntryList = {
            { 
                Polygon = polygon.createCircle( 2.0, {0, 0}, 9 ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true }
			},
		}
	},
	Cost = {
		BuildRightTaxes = { { Resource = "GOLD", Quantity = 5 } },
		UpkeepCost = { { Resource = "GOLD", Quantity = 5 } },
		ResourceNeededList = {
            {
                { Resource = "WOOD", Quantity = 2 },
                { Resource = "TOOL", Quantity = 1 },
            },
		},
    },
})



-- Job: Rubbish Collector -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "JOB",
	Id = "JOB_RUBBISH_COLLECTOR",
	JobName = "JOB_RUBBISH_COLLECTOR_NAME",
	JobDescription = "JOB_RUBBISH_COLLECTOR_DESC",
	IsLockedByDefault = false,
	DefaultBehavior = "BEHAVIOR_GATHER",
    ProductionDelay = 6.5, -- default 30
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
	CharacterSetup = {
		DataType = "CHARACTER_SETUP",
        RightHandObject = {
            Model = "PREFAB_TOOL_BLACKSMITH_HAMMER"
        },
		IdleAnimation = "MINE",
	},
})

mod:overrideAsset({
    Id = "NEWCOMER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_RUBBISH_COLLECTOR"
    }
})

mod:overrideAsset({
    Id = "SERF",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_RUBBISH_COLLECTOR"
    }
})



-- Building Function: Rubbish Collector -------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_WORKPLACE",
    Id = "BUILDING_FUNCTION_RUBBISH_COLLECTOR",
    Name = "BUILDING_FUNCTION_RUBBISH_COLLECTOR_NAME",
    WorkerCapacity = 1,
    RelatedJob = {
        Job = "JOB_RUBBISH_COLLECTOR",
        Behavior = "BEHAVIOR_GATHER"
    },
    UpkeepPerCapacity = {
		{ Resource = "GOLD_COINS", Quantity = 2 },
    },
    InputInventoryCapacity = {
	},
    ResourceListNeeded = {
		{ Resource = "RESOURCE_RUBBISH", Quantity = 1 },
	},
    ResourceProduced = {
		{ Resource = "WOOD", Quantity = 4 },
		{ Resource = "STONE", Quantity = 2 },
		{ Resource = "IRON", Quantity = 1 },
	}
})




-- *************************************************************************************************************
-- Rubbish Controller -------------------------------------------------------------------------------------
-- Creates and removes rubbish from the settlement automatically
-- *************************************************************************************************************

local COMP_RUBBISH_CONTROLLER = {
	TypeName = "COMP_RUBBISH_CONTROLLER",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "ListRubbish", Type = "list<string>", Default = nil, Flags = { "SAVE_GAME" } },
    }
}


-- Init
function COMP_RUBBISH_CONTROLLER:onEnabled()


    -- Daily script
    local compMainGameLoop = self:getLevel():find("COMP_MAIN_GAME_LOOP")
    event.register(self, compMainGameLoop.ON_NEW_DAY, function()

        -- mod:log("Daily rubbish event")

        -- Get and shuffle a list of enabled villagers
        local listCompVill = {}
        self:getLevel():getComponentManager("COMP_VILLAGER"):getAllEnabledComponent():forEach(function(compVillager)
            table.insert(listCompVill, compVillager)
        end)

        ShuffleTable(listCompVill) -- from function.lua


        -- Pick a % of villagers who will leave Rubbish
        local popVill = #listCompVill
        local countPickVill = math.random(popVill * 0.00, popVill * 0.04)
        -- local countPickVill = popVill -- TESTING ONLY
        if countPickVill < 1 then countPickVill = 1 end


        -- Iterate through shuffled villager list up to the picked count
        for i = 1, countPickVill do

            -- Get the villager object
            local objVill = listCompVill[i]:getOwner()
            -- mod:log("Villager will leave rubbish: " .. tostring(objVill.Name))

            -- Pick a nearby position
            local posVill = objVill:getGlobalPosition()
            local posRubbish = { posVill.x + math.random(-4, 4), 0, posVill.z + math.random(-4, 4) }

            -- Pick one of the rubbish prefabs
            local pickRubbishPrefab = math.random(1, #listRubbishPrefab)

            -- Create a new rubbish prefab at the chosen position, rotate and scale randomly
            local objRubbish = self:getLevel():createObject(listRubbishPrefab[pickRubbishPrefab], posRubbish)
            if objRubbish then

                objRubbish:setRotationY(2 * math.pi * math.random())
                local scaleX = math.random(80, 110) / 100
                local scaleY = math.random(80, 110) / 100
                objRubbish:scale( { scaleX, scaleY, scaleX } )

                -- Record the new rubbish object
                table.insert(self.ListRubbish, objRubbish:getId())

            end

            -- Delete one rubbish object from the beginning of the list, if the list is past the max amount (depends on population)
            if #self.ListRubbish > (popVill * 2) then
                local objDestroy = self:getLevel():find(self.ListRubbish[1])
                table.remove(self.ListRubbish, 1)
                if objDestroy then
                    objDestroy:destroy()
                end
            end

        end

    end)

end


-- Register custom Class
mod:registerClass(COMP_RUBBISH_CONTROLLER)


-- Attach class to prefab
mod:registerPrefabComponent("PREFAB_MANAGER", { DataType = "COMP_RUBBISH_CONTROLLER" } )



